import { useRef } from 'react';
import { DeviceEventEmitter } from 'react-native';
import { useLoad, useUnload } from 'react-native-lifecycle';
/**
 * 事件订阅
 * @param {string} eventType 事件名称
 * @param {function} effect 订阅事件
 * @public
 */
export default (eventType, effect) => {
    const emitterSubscription = useRef(null);
    useLoad(() => {
        if (typeof effect === 'function') {
            emitterSubscription.current = DeviceEventEmitter.addListener(eventType, effect);
        }
    });
    useUnload(() => {
        var _a;
        if (typeof effect === 'function') {
            if (typeof DeviceEventEmitter.removeListener === 'function') {
                DeviceEventEmitter.removeListener(eventType, effect);
            }
            else {
                (_a = emitterSubscription.current) === null || _a === void 0 ? void 0 : _a.remove();
            }
        }
    });
    return {
        emit: (...params) => DeviceEventEmitter.emit(eventType, ...params),
    };
};
